/**
 * Generic class for a Conditional object.
 */
var EddCmConditional = (function EddCmConditionalClass() {

	// Inheritance
	EddCmConditional.prototype = Object.create(Object.prototype);

	/**
	 * jQuery alias.
	 * @type {jQuery}
	 */
	var $ = jQuery;

	/**
	 * Constructor.
	 */
	function EddCmConditional( conditions, state ) {
		this.conditions = [];

        this.state = (typeof state === 'undefined')
			? {}
			: state;
        this.state.children = (typeof this.state.children === 'undefined')
			? {}
			: this.state.children;

        this.initElem();
        this.initEvents();

		conditions = typeof conditions === 'undefined'? [] : conditions;

		for (var i in conditions) {
			this.addCondition( conditions[i], this.state.children[i] );
        }

		this.updateTogglButton();
	}
	
	/**
	 * Initializes the element.
	 */
	EddCmConditional.prototype.initElem = function() {
		// Clone the template element
		this.elem = $('#edd-cm-conditional-template').clone(true, true).children();
		// Set pointers to first-level children elements
		this.contentContainer = this.elem.find('> div.content');
		this.buttonsContainer = this.elem.find('> div.buttons');
		this.toggleable = this.elem.find('> div.toggleable-section');
		// Set pointers to conditions list and labels
		this.conditionsList = this.toggleable.find('> ul.conditions');
		this.labelsContainer = this.toggleable.find('> div.condition-labels');
		// Set button pointers
		this.toggleConditionsButton = this.buttonsContainer.find('> button.toggle-conditions-button');
		this.deleteButton = this.buttonsContainer.find('> button.delete-button');
		this.addConditionButton = this.toggleable.find('> button.add-condition-button');

        if (this.state.closed === true) {
        	this.toggleable.hide();
        	this.toggleButtons();
        }
	};

	/**
	 * Adds a condition.
	 */
	EddCmConditional.prototype.addCondition = function( obj, state ) {
		var condition = new EddCmCondition(
			obj.type,
			obj.negation,
			obj.data,
			obj.conditions,
			state
		);
		// Add condition to array
		this.conditions.push( condition );
		// Save new index: index of last element at this point in time
		var index = this.conditions.length - 1;
		// Add element
		this.conditionsList.append( condition.elem );
		// On remove event of condition
		condition.elem.on('edd_cm_remove', function(event) {
			this.removeCondition( index );
			event.stopPropagation();
		}.bind(this));
		// Run conditions changed method
		this.onConditionsChanged();
	};

	/**
	 * Removes a condition.
	 */
	EddCmConditional.prototype.removeCondition = function( index ) {
		this.conditions.splice(index, 1);
		// Run conditions changed method
		this.onConditionsChanged();
	};

	/**
	 * Runs after a condition has been added/removed
	 */
	EddCmConditional.prototype.onConditionsChanged = function() {
		var numConditions = this.conditions.length;
		this.labelsContainer.find('p.has-conditions').toggle( numConditions > 0 );
		this.labelsContainer.find('p.no-conditions').toggle( numConditions == 0 );
		this.updateTogglButton();
	};

	/**
	 * Updates the toggle (collapse/expand) button, based on whether this conditional entry
	 * has children conditions.
	 */
	EddCmConditional.prototype.updateTogglButton = function() {
		this.toggleConditionsButton.prop('disabled', this.conditions.length == 0);
	};

	/**
	 * Initializes events for HTML elements.
	 */
	EddCmConditional.prototype.initEvents = function() {
		this.toggleConditionsButton.unbind('click').click( this.toggleState.bind(this) );
		this.deleteButton.unbind('click').click( this.onDeleteClick.bind(this) );
		this.addConditionButton.unbind('click').click( this.onAddConditionClick.bind(this) );
		this.onConditionsChanged();
	};

	/**
	 * CAllback function called on the clicked event for the add condition button.
	 */
	EddCmConditional.prototype.onAddConditionClick = function() {
		this.addCondition( new EddCmCondition() );
	};

	/**
	 * Callback function called on the click event for the toggle conditions button.
	 */
	EddCmConditional.prototype.toggleState = function() {
		this.toggleable.slideToggle(300, function() {
			this.toggleButtons();
		}.bind(this));

		this.state.closed = !this.state.closed;
	};

    /**
	 * Toggles the state of buttons for the conditional.
     */
	EddCmConditional.prototype.toggleButtons = function() {
        this.toggleConditionsButton
			.add(this.labelsContainer)
			.find('> i.fa')
			.toggleClass('fa-caret-down')
			.toggleClass('fa-caret-up');
	};

	/**
	 * Callback function called on the click event for the delete button.
	 */
	EddCmConditional.prototype.onDeleteClick = function() {
		this.destroy();
	};

    /**
     * Gets the state.
     */
    EddCmConditional.prototype.getState = function() {
		var a = {
			closed: this.state.closed,
			children: {}
		};

		for (var i in this.conditions) {
			a.children[i] = this.conditions[i].getState();
		}

		return a;
    };

	/**
	 * Destructor.
	 */
	EddCmConditional.prototype.destroy = function() {
		this.elem.trigger('edd_cm_remove').remove();
	};

	/**
	 * Converts into an std object.
	 */
	EddCmConditional.prototype.toObject = function() {
		return {
			conditions: _.map(this.conditions, function(condition) {
				return condition.toObject();
			})
		};
	};

	return EddCmConditional;
}());
