/**
 * Class for messages list.
 */
var EddCmMessagesList = (function EddCmMessagesListClass(top, undefined) {
	
	// Inheritance
	EddCmMessagesList.prototype = Object.create(Object.prototype);

	/**
	 * jQuery alias.
	 * @type {jQuery}
	 */
	var $ = jQuery;

	var collapsedStateCookieName = 'eddCmSettingsCollapsedState';

	/**
	 * Constructor.
	 *
	 * @param {jQuery|DomElement} elem
	 * @param {Array}             messages
	 */
	function EddCmMessagesList( elem, storage, cookie ) {
		// Save key element pointers
		this.elem = $(elem);
		this.storage = $(storage);
		// Load saved data from storage element
		var saved = JSON.parse(this.storage.val());
		// Ensure there is at least one message
		if ( saved.length === 0 ) saved.push( new EddCmMessage() );

        // Get collapsed state
        this.cookie = cookie;
        this.state = this.cookie.getJSON(collapsedStateCookieName);
        if (typeof this.state !== 'object') {
            this.state = {};
        }

		// Initialize
		this.initElem();
		this.initEvents();
		// Add messages from saved data
		this.messages = [];
		for (var i in saved) {
            this.addMessage(saved[i], this.state[i]);
        }
		// Update the "no messages" elem
		this.updateNoMessagesElement();
	}

	/**
	 * Initializes the element.
	 */
	EddCmMessagesList.prototype.initElem = function() {
		// Cone the template
		this.elem.append( this.cloneTemplate() );
		// Reset elem pointer to inner element
		this.elem = this.elem.find('> .edd-cm-messages-container');
		// Save a pointer to the messages list element
		this.msgListElem = this.elem.find('> ul.messages');
		// Save pointe to the add message button
		this.addMessageButton = this.elem.find('> button.add-message-button');
		// Save pointer to the "no messages" element
		this.noMessagesElem = this.elem.find('> p.edd-cm-no-messages');
		// Add any messages saved in this instance
		for( var i in this.messages ) {
			this.addMessage(this.messages[i], this.state[i]);
        }
	};

	/**
	 * Clones the template into a new element and returns it.
	 */
	EddCmMessagesList.prototype.cloneTemplate = function() {
		return $('#edd-cm-messages-container-template').children().clone(true, true);
	};

	/**
	 * Initializes events in the element and its children.
	 */
	EddCmMessagesList.prototype.initEvents = function () {
		this.addMessageButton.unbind('click').click( this.onAddMessageButtonClick.bind(this) );
		$('form').submit( this.onFormSubmit.bind(this) );

        $(window).on('unload', this.saveCollapsedState.bind(this));
	};

	/**
	 * Adds a new message.
	 */
	EddCmMessagesList.prototype.addMessage = function( msgObj, state ) {
		var message = new EddCmMessage(
			msgObj.message,
			msgObj.style,
			msgObj.conditions,
			state
		);
		// Add message to array
		this.messages.push( message );
		// Save pushed index: index of last element at this point in time
		var index = this.messages.length - 1;
		// Add element
		this.msgListElem.append( message.elem );
		// On remove event
		message.elem.on('edd_cm_remove', function(event) {
			this.removeMessage(index);
		}.bind(this));
		this.messagesChanged();
	};

	/**
	 * Removes a message.
	 */
	EddCmMessagesList.prototype.removeMessage = function( index ) {
		this.messages.splice(index, 1);
		this.messagesChanged();
	};

	/**
	 * Executed when a message is added/removed.
	 */
	EddCmMessagesList.prototype.messagesChanged = function() {
		this.updateNoMessagesElement();
	}

	/**
	 * Shows/hides the "no messages" element depending on whether there are messages in the list.
	 */
	EddCmMessagesList.prototype.updateNoMessagesElement = function() {
		this.noMessagesElem.toggle(this.messages.length == 0);
	}

	/**
	 * Gets the messages.
	 * 
	 * @return {Array}
	 */
	EddCmMessagesList.prototype.getMessages = function() {
		return this.messages;
	};

	/**
	 * Callback for click event on the add message button.
	 */
	EddCmMessagesList.prototype.onAddMessageButtonClick = function( event ) {
		this.addMessage( new EddCmMessage() );
	};

	EddCmMessagesList.prototype.onFormSubmit = function (event) {
		this.storage.val( JSON.stringify( this.toArray() ) );
	};

    /**
     * Converts into an array.
     */
    EddCmMessagesList.prototype.getState = function() {
    	var a = {};

        for ( var i in this.messages ) {
        	a[i] = this.messages[i].state;
		}

		return a;
    };

	/**
	 * Converts into an array.
	 */
	EddCmMessagesList.prototype.saveCollapsedState = function() {
		var s = this.getState();
		console.log(s);
		this.cookie.set(collapsedStateCookieName, s, {
			expires: 365
		});
	};

	/**
	 * Converts into an array.
	 */
	EddCmMessagesList.prototype.toArray = function() {
		return _.map(this.messages, function(message) {
			return message.toObject();
		});
	};

	// Return the class type
	return EddCmMessagesList;
}(top));
