<?php

namespace EddCm\Controllers;
use EddCm\Main;

/**
 * AJAX controller class.
 *
 * An AJAX request from this plugin uses the following `data` structure:
 * data: {
 * 		action:	'wordpress_action_hook_name',
 * 		edd_cm: {
 * 			command: 'edd_cm_command',
 * 			args: { .. arguments .. }
 * 		}
 * }
 */
class AjaxController extends AbstractController {

	/**
	 * The registered command handlers.
	 * 
	 * @var array
	 */
	protected $_handlers;

	/**
	 * Constructor.
	 */
	public function __construct() {
		parent::__construct('Ajax');
		$this->_handlers = array();
		do_action( 'edd_cm_ajax_controller_loaded', $this );
	}

	/**
	 * Initializes the controller.
	 */
	public function init() {
		$hooks = Main::getInstance()->getHookLoader();
		$hooks->add_action( 'wp_ajax_edd_cm_ajax', $this, 'onAjaxRequest' );
	}

	/**
	 * Registers a command handler.
	 * 
	 * @param  string $command      The AJAX command.
	 * @param  mixed  $callbackArg1 The callback function name, class name or object instance.
	 * @param  mixed  $callbackArg2 The method or static function name if $callbackArg1 is a class name or object instance.
	 */
	public function registerHandler( $command, $callbackArg1, $callbackArg2 = null ) {
		$callback = ( $callbackArg2 === null )? $callbackArg1 : array( $callbackArg1, $callbackArg2 );
		$this->_handlers[ $command ] = $callback;
	}

	/**
	 * Checks if a handler is registered for a specific command.
	 * 
	 * @param  string  $command The command.
	 * @return boolean          True if a handler is registered for this given command, false otherwise.
	 */
	public function hasHandlerForCommand( $command ) {
		return isset( $this->_handlers[ $command ] );
	}

	/**
	 * Gets the callback for a specific command's handler.
	 * 
	 * @param  string $command The command
	 * @return callable        The callback for the handler registered for the given command.
	 */
	public function getHandlerCallback( $command ) {
		return $this->_handlers[ $command ];
	}

	public function getHandlers() {
		return $this->_handlers;
	}

	/**
	 * Executes the callback for the handler registered for a specific command.
	 * 
	 * @param  string $command The command.
	 * @param  array $args     The arguments to pass to the callback. Default: array().
	 */
	public function invokeHandler( $command, $args = array() ) {
		$callback = $this->getHandlerCallback( $command );
		call_user_func_array( $callback, $args );
	}

	/**
	 * Method called when an AJAX request is received.
	 */
	public function onAjaxRequest() {
		// Look for the edd_cm index adn command subindex
		if ( !isset($_POST['edd_cm'], $_POST['edd_cm']['command']) ) return;

		// Get subindexes
		$data = $_POST['edd_cm'];
		$command = str_replace( '\\\\', '\\', $data['command'] );
		$args = isset( $data['args'] )? $data['args'] : array();
		
		// Look for a handler for this command and invoke its callback
		if ( $this->hasHandlerForCommand( $command ) )
			$this->invokeHandler( $command, $args );
	}

}

// Create a new controller instance
new AjaxController();
