<?php

namespace EddCm\Controllers;
use \EddCm\Main as Main;
use \EddCm\FileLoader as FileLoader;

/**
 * Settings controller class.
 */
class SettingsController extends AbstractController {

	const SLUG = 'eddcm';

	/**
	 * Constructor.
	 */
	public function __construct() {
		parent::__construct('Settings');
	}

	/**
	 * Initializes the controller.
	 */
	public function init() {
		$hooks = Main::getInstance()->getHookLoader();
		$hooks->add_action( 'admin_enqueue_scripts', $this, 'enqueue_styles' );
		$hooks->add_action( 'admin_enqueue_scripts', $this, 'enqueue_scripts' );
		$hooks->add_filter( 'edd_settings_sections_extensions', $this, 'register_tab' );
		$hooks->add_filter( 'edd_settings_extensions', $this, 'register_settings' );
		$hooks->add_filter( 'edd_settings_extensions_sanitize', $this, 'sanitize_settings' );
		$hooks->add_action( 'edd_cm_messages', $this, 'render_messages_settings' );
        $hooks->add_action( 'edd_cm_reset', $this, 'render_reset_settings' );
	}

	/**
	 * Gets the saved value from DB.
	 * 
	 * @return mixed
	 */
	public function get() {
		$edd_settings = get_option('edd_settings');
		$messages = isset($edd_settings['cm_messages'])
			? $edd_settings['cm_messages']
			: array();

        return is_string($messages)
            ? json_decode($messages, true)
            : $messages;
	}

	/**
	 * Registers the tab in the EDD settings page.
	 * 
	 * @param  array $tabs The original EDD tabs array
	 * @return array       The tabs with the added messages tab
	 */
	public function register_tab( $tabs ) {
		$tabs[self::SLUG] = self::get_tab_label();

		return $tabs;
	}

	/**
	 * Registers the settings.
	 * 
	 * @param  array $settings The original EDD settings array.
	 * @return array           The settings with the added messages settings
	 */
	public function register_settings( $settings ) {
		$settings[static::SLUG] = array(
			'cm_messages' => array(
				'id'		=>	'cm_messages',
				'name'		=>	__( 'Messages', MAIN::TEXT_DOMAIN ),
				'desc'		=>	__( 'The checkout messages.', MAIN::TEXT_DOMAIN ),
				'type'		=>	'hook',
			),
            'cm_reset'   => array(
                'id'       => 'cm_reset',
                'name'     => __('Reset Settings', 'eddcm'),
                'desc'     => __('Reset your message settings back to default', 'eddcm'),
                'type'     => 'hook'
            )
		);
		return $settings;
	}

	/**
	 * Renders the message settings.
	 * 
	 * @param  array $args View arguments.
	 */
	public function render_messages_settings( $args ) {
		echo Main::renderView('Settings');
	}

    /**
	 * Renders the reset settings option.
	 *
	 * @param  array $args View arguments.
	 */
	public function render_reset_settings( $args ) {
		echo Main::renderView('ResetSettings');
	}

	/**
	 * Sanitizes the settings.
	 * 
	 * @param  [type] $value [description]
	 * @param  [type] $key   [description]
	 * @return [type]        [description]
	 */
	public function sanitize_settings($input) {
        $output = $input;

        if (isset($input['cm_messages'])) {
            $options = $input['cm_messages'];
            if (is_array($options) && !empty($options['cm_reset'])) {
                $output['cm_messages'] = array();
            } else {
                $output['cm_messages'] = is_string($options)
                    ? json_decode($options, true)
                    : $options;
            }
        }

        return $output;
	}

	/**
	 * Checks if the current page is the messages settings page.
	 * 
	 * @return boolean True if current page is the messages settings page, false otherwise.
	 */
	protected static function _isMessagesSettingsPage() {
		$screen = \get_current_screen();
		$tab = isset($_GET['tab'])
            ? $_GET['tab']
            : '';

		return $screen !== null && ($screen->id === 'download_page_edd-settings' && $tab === 'extensions');
	}

	/**
	 * Registers styles to WordPress.
	 */
	public function enqueue_styles() {
		if ( ! self::_isMessagesSettingsPage() ) return;
        $version = Main::getInstance()->getData('Version');
		wp_enqueue_style(
            'edd-cm-fa-css',
            FileLoader::uri(Main::ASSETS_PATH, 'css', 'font-awesome.min.css'),
            array(),
            $version
        );
		wp_enqueue_style(
            'edd-cm-settings-css',
            FileLoader::uri(Main::ASSETS_PATH, 'css', 'edd-cm-settings.css'),
            array('wp-admin', 'buttons'),
            $version
        );
        wp_enqueue_style( 'jquery-chosen' );
    }

	/**
	 * Registers scripts to WordPress.
	 */
	public function enqueue_scripts() {
		if ( ! self::_isMessagesSettingsPage() ) return;
        $version = Main::getInstance()->getData('Version');
        wp_enqueue_script(
            'edd-cm-cookie-js',
            FileLoader::uri( Main::ASSETS_PATH, 'js', 'js.cookie-2.1.4.min.js' ),
            array(),
            '2.1.4'
        );
		wp_enqueue_script(
            'edd-cm-conditional-class-js',
            FileLoader::uri( Main::ASSETS_PATH, 'js', 'edd-cm-conditional-class.js' ),
            array(),
            $version
        );
		wp_enqueue_script(
            'edd-cm-condition-class-js',
            FileLoader::uri( Main::ASSETS_PATH, 'js', 'edd-cm-condition-class.js' ),
            array(),
            $version
        );
		wp_enqueue_script(
            'edd-cm-message-class-js',
            FileLoader::uri( Main::ASSETS_PATH, 'js', 'edd-cm-message-class.js' ),
            array(),
            $version
        );
		wp_enqueue_script(
            'edd-cm-messages-list-class-js',
            FileLoader::uri( Main::ASSETS_PATH, 'js', 'edd-cm-messages-list-class.js' ),
            array(),
            $version
        );
		wp_enqueue_script(
            'edd-cm-settings-js',
            FileLoader::uri( Main::ASSETS_PATH, 'js', 'edd-cm-settings.js' ),
            array(),
            $version
        );
        wp_enqueue_script('jquery-chosen');
    }

	/**
	 * Gets the tab label.
	 * 
	 * @return string
	 */
	public static function get_tab_label() {
		return __( 'Checkout Messages', 'eddcm' );
	}

}

// Create a new controller instance
new SettingsController();
