<?php

namespace EddCm\Lib\Actions;

use \EddCm\Main;
use \EddCm\Lib\Base\AbstractAction;
use \EddCm\Lib\Base\AbstractCondition;
use \EddCm\Lib\Base\AbstractStyle;

/**
 * Action class to show a message when executed.
 */
class MessageAction extends AbstractAction {

	/**
	 * The default view for style-less messages.
	 */
	const DEFAULT_VIEW = 'Message';

	/**
	 * The text of the message.
	 * 
	 * @var string
	 */
	protected $_message;

	/**
	 * The classname of the style to use for this message.
	 * 
	 * @var string
	 */
	protected $_style;

	/**
	 * Constructor.
	 * 
	 * @param string $message    The message to display when the action is executed.
	 * @param array  $conditions The conditions
	 */
	public function __construct( $message, $style = '', $conditions = array() ) {
		parent::__construct( $conditions );
		$this->setMessage( $message );
		$this->setStyle( $style );
	}

	/**
	 * Sets the message text.
	 * 
	 * @param string $message
	 */
	public function setMessage( $message ) {
		$this->_message = $message;
	}

	/**
	 * Gets the message text.
	 * 
	 * @return string
	 */
	public function getMessage() {
		return $this->_message;
	}

	/**
	 * Sets the style of the message.
	 *
	 * @param string $style The classname of the style.
	 * @return  self        This instance.
	 */
	public function setStyle( $style ) {
		if ( is_subclass_of( $style, 'EddCm\Lib\Base\AbstractStyle' ) ) {
			if ( is_string( $style ) ) $style = new $style();
			$this->_style = $style;
		}
		return $this;
	}

	/**
	 * Gets the style of the message.
	 * 
	 * @return string The classname of the style.
	 */
	public function getStyle() {
		return $this->_style;
	}

	/**
	 * Internal exec method. Shows the message.
	 * 
	 * @param  array  $args The arguments.
	 */
	protected function _exec($args = array()) {
		$text = $this->prepareMessageText($args);
		if ( is_object( $this->_style ) ) {
			return $this->_style->renderView( $text );
		} else {
			return Main::renderView( self::DEFAULT_VIEW, compact('text') );
		}
	}

	/**
	 * Prepares the message text and returns it.
	 * 
	 * @param  array  $args Optional arguments to use for generating the text. Default: array()
	 * @return string       The message text.
	 */
	public function prepareMessageText($args = array()) {
		$escapedMessage = str_replace('%', '%%', $this->_message);
		return vsprintf($escapedMessage, $args);
	}

	/**
	 * Creates a new instance from the given data array.
	 * 
	 * @param  array         $arr The data array to use to create the message action instance.
	 *                            Indexes used:
	 *                              - message ........ The message text
	 *                              - conditions ..... Array of conditions (either instances or in array form)
	 * @return MessageAction      The created instance.
	 */
	public static function fromArray( $arr ) {
		$message = $arr['message'];
		$style = isset( $arr['style'] )? $arr['style'] : '';
		$conditions = array();
		foreach ( $arr['conditions'] as $condition )
			if ( is_array( $condition ) ) {
				// Get condition from array
				$newCondition = AbstractCondition::fromArray( $condition );
				// Add, if not null
				if ($newCondition !== null) {
					$conditions[] = $newCondition;
				}
			}
			else $conditions[] = $condition;
		return new MessageAction( $message, $style, $conditions );
	}

}
