<?php

namespace EddCm\Lib\Base;
use EddCm\Main;

/**
 * Abstract class for a condition.
 */
abstract class AbstractCondition extends AbstractConditionalObject implements Condition {

	/**
	 * Negation flag, used to determine the polarity of the condition.
	 * 
	 * @var boolean
	 */
	protected $_negation;

	/**
	 * Constructs a new instance.
	 * 
	 * @param array $conditions Subconditions array.
	 */
	public function __construct( $conditions = array() ) {
		parent::__construct( $conditions );
		$this->_negation = false;
	}

	/**
	 * Checks if the condition is negated.
	 * 
	 * @return boolean True if the condition is negated and false if not.
	 */
	public function isNegated() {
		return $this->_negation;
	}

	/**
	 * Sets the condition's negation.
	 * 
	 * @param boolean $negation True for default polarity, false for negated.
	 */
	public function setNegation( $negation ) {
		$this->_negation = ($negation == true);
	}

	/**
	 * Gets the boolean value that the condition's _evaluate method must return for the condition
	 * to be satisifed.
	 * 
	 * @return boolean
	 */
	public function getEvaluationSatisfactionValue() {
		return ! $this->isNegated();
	}

	/**
	 * Evaluation method that checks if this condition is satisfied or not.
	 * 
	 * @return boolean True if the condition is satisfied, false otherwise.
	 */
	public function evaluate() {
		return $this->evaluateConditions() && ( $this->_evaluate() === $this->getEvaluationSatisfactionValue() );
	}
	
	/**
	 * Registers the condition to the ConditionsController.
	 * Should be called on hook:action:edd_cm_controllers_loaded
	 */
	public static function register() {
		Main::getInstance()->getController('Conditions')->registerConditionType( get_called_class() );
		static::_afterRegister();
	}

	/**
	 * Internal static method to execute after the condition is registered.
	 */
	protected static function _afterRegister() {
		$ajaxCommand = sprintf( 'get_condition_fields[%s]', get_called_class() );
		Main::getInstance()->getController('Ajax')->registerHandler( $ajaxCommand, get_called_class(), 'renderFieldsAjax' );
	}

	/**
	 * Renders the fields for this condition.
	 *
	 * @return string
	 */
	public static function renderFields() {
		$viewName = constant( get_called_class() . '::FIELD_TYPE' );
		return Main::renderView( $viewName, static::_getViewbag() );
	}

	/**
	 * Returns the viewbag for the fields view.
	 * 
	 * @return array
	 */
	protected static function _getViewbag() {
		return array();
	}

	/**
	 * Echoes a JSON response containing the fields for this condition and dies.
	 */
	public static function renderFieldsAjax() {
		echo json_encode(
			array( 'fields'	=>	static::renderFields() )
		);
		die;
	}

	/**
	 * Internal evaluation function.
	 * 
	 * @return boolean True if the condition is satisfied, false otherwise.
	 */
	abstract protected function _evaluate();

	/**
	 * Creates a condition from the given array of data.
	 * 
	 * @param  array $arr Array containing the data that will be used to create the condition instance.
	 *                    Indexes used:
	 *                    	type ......... The condition type (class name)
	 *                    	negation ..... The negation flag
	 *                    	data ......... Array of data related to the condition
	 *                    	conditions ... The children conditions.
	 * @return mixed      Condition instance, the class of which depends on the 'type' index of the given array.
	 */
	public static function fromArray( $arr ) {
		// Get the condition class, negation and data
		$class = $arr['type'];
		// If the class is empty, stop
		if (!is_string($class) || strlen($class) === 0) return null;
		$negation = $arr['negation'];
		$data = $arr['data'];
		// Prepare the children conditions
		$conditions = array();
		foreach ( $arr['conditions'] as $conditionArray )
			$conditions[] = self::fromArray( $conditionArray );
		// Create the instance
		$instance = new $class($conditions);
		// Set the negation
		$instance->setNegation( $negation );
		// Set the data
		foreach ( $data as $key => $value ) {
			// Change snake case $value to camel case and generate the setter method name
			$camelCase = implode( '', array_map( 'ucfirst', explode( '_', $key ) ) );
			$methodName = sprintf( 'set%s', $camelCase );
			// If the method exists, set the value
			if ( method_exists( $instance, $methodName ) )
				$instance->$methodName( $value );
		}
		// Return the finished instance
		return $instance;
	}

}
