<?php

namespace EddCm;
use EddCm\Controllers\AbstractController;
use EddCm\Controllers\SettingsController;
use EddCm\Controllers\AjaxController;
use EddCm\Controllers\ConditionsController;
use EddCm\Controllers\OperatorsController;
use EddCm\Controllers\MessagesController;
use EddCm\Exceptions\SingletonReinstantiation;
use EDD_License;

/**
 * Main plugin class.
 *
 * This class acts as the central main hub for all other classes and components that make up the plugin.
 * The first-level components attached to this class must be controllers that reserve a portion of the plugin's
 * structure, logic and/or interface.
 *
 * Main
 *  |- Controller
 *  |   |- [other classes]
 *  |   '- ...
 *  |- Controller
 *  |   '- ...
 *  |- ...
 *  '- Controller
 */
class Main {

	// Text domain constant
	const TEXT_DOMAIN = 'eddcm';

	// Path constants
	const ASSETS_PATH = 'assets';
	const INCLUDES_PATH = 'includes';
	const CLASSES_PATH = 'classes';
	const CONTROLLERS_PATH = 'controllers';
	const VIEWS_PATH = 'views';
	const LANG_PATH = 'languages';

	/**
	 * The plugin meta data.
	 * 
	 * @var array
	 */
	protected $_data;

	/**
	 * The EDD extension license.
	 * 
	 * @var EDD_License
	 */
	protected $_license;

	/**
	 * The internationalization class instance.
	 * 
	 * @var EddCm\I18n
	 */
	protected $i18n;

	/**
	 * The hook loader.
	 * 
	 * @var EddCm\Wp\HookLoader
	 */
	protected $_hooks;

	/**
	 * Registered controllers.
	 *
	 * @var array
	 */
	protected $_controllers;

	/**
	 * The singleton instance.
	 * 
	 * @var self
	 */
	protected static $_instance;

	/**
	 * Constructs the singleton instance.
	 */
	public function __construct() {
		// If the singleton instance is already instansiated
		if ( self::$_instance !== NULL )
			// Throw exception. Since the class has already been instantiated at this point, the exception class
			// should already exist and does not require loading.
			throw new SingletonReinstantiation( __CLASS__ );
		// Else set the singleton to this instance
		else self::$_instance = $this;

		// Retrieve the plugin meta data
		$this->_loadPluginData();
		
		// Set up the license handler
		if ( class_exists( 'EDD_License' ) ) {
			$this->_license = new EDD_License( EDDCM_MAIN_FILE, EDD_CM_EXTENSION_NAME, $this->getData('Version'), $this->getData('Author') );
		}

		// Init the I18n instance
		$this->i18n = new I18n( self::TEXT_DOMAIN );
		$this->i18n->loadTextDomain();

		// Init hook loader
		$this->_hooks = new Wp\HookLoader();
		do_action( 'edd_cm_hook_loader_ready' );

		// Initialize the controllers
		$this->_initControllers();
		
		// Hot-load lib files
        FileLoader::loadDirectory(EDDCM_INCLUDES_DIR . 'Lib/Styles');
		FileLoader::loadDirectory(EDDCM_INCLUDES_DIR . 'Lib/Styles');
		FileLoader::loadDirectory(EDDCM_INCLUDES_DIR . 'Lib/Actions');
		FileLoader::loadDirectory(EDDCM_INCLUDES_DIR . 'Lib/Conditions');
		FileLoader::loadDirectory(EDDCM_INCLUDES_DIR . 'Lib/Operators');

		// Call controllers' `init`
		foreach ( $this->_controllers as $handle => $controller )
			$controller->init();

		// Controllers loaded action
		do_action( 'edd_cm_controllers_loaded' );
	}

	/**
	 * Intializes the controllers.
	 */
	protected function _initControllers() {
		$this->_controllers = array();
		// Required controllers
		$this->addControllers(
			array(
				'Ajax'			=>	new AjaxController(),
				'Settings'		=>	new SettingsController(),
				'Messages'		=>	new MessagesController(),
				'Conditions'	=>	new ConditionsController(),
				'Operators'		=>	new OperatorsController()
			)
		);
	}

	/**
	 * Registers a set of controllers.
	 * 
	 * @param array $arr An array of controllers. Array keys must be the handles and values must be
	 *                   the controller instances.
	 */
	public function addControllers( $arr ) {
		foreach ( $arr as $handle => $controller )
			$this->addController( $handle, $controller );
	}

	/**
	 * Registers a controller.
	 * 
	 * @param string $handle     Unique string handle to identify and reference the controller.
	 * @param mixed  $controller A controller, instance of a subclass of AbstractController..
	 */
	public function addController( $handle, AbstractController $controller ) {
		// Add the controller
		$this->_controllers[ $handle ] = $controller;
	}

	/**
	 * Removes the controller with the given handle.
	 * 
	 * @param  string  $handle The handle of the controller to remove.
	 * @return boolean         True if the controller was removed, false if it was not found.
	 */
	public function removeController( $handle ) {
		if ( ! $this->hasControllers( $handle ) )
			return false;
		unset( $this->_controllers[ $handle ] );
		return true;
	}

	/**
	 * Gets the controller with the given handle.
	 *
	 * @param  string $handle The handle of the controller to retrieve.
	 * @return mixed          The controller object with the given handle, or null if not found.
	 */
	public function getController( $handle ) {
		return $this->hasControllers( $handle )? $this->_controllers[ $handle ] : null;
	}

	/**
	 * Gets all of the registered controllers.
	 * 
	 * @return array
	 */
	public function getControllers() {
		return $this->_controllers;
	}

	/**
	 * Checks if a controller with the given handle is registered.
	 * 
	 * @return boolean True if the controller is registered, false otherwise.
	 */
	public function hasControllers( $handle ) {
		return isset( $this->_controllers[ $handle ] );
	}

	/**
	 * Gets the hook loader.
	 * 
	 * @return EddCm\Wp\HookLoader
	 */
	public function getHookLoader() {
		return $this->_hooks;
	}

	/**
	 * Gets the plugin data, a single data entry or a given default as a fallback if given.
	 * 
	 * @param  string $key     The key of the data entry to return. Default: null
	 * @param  mixed  $default The value to return if the data entry is not found. Default: null
	 * @return mixed           If the $key arg is null, the entire data set is returned. Otherwise, the
	 *                         data entry with that key will be returned, or $default if not found.
	 */
	public function getData( $key = null, $default = null ) {
		if ( $key === null )
			return $this->_data;
		if ( isset( $this->_data[ $key ] ) )
			return $this->_data[ $key ];
		return $default;
	}

	/**
	 * Attaches all hooks to WordPress to prepare for execution.
	 */
	public function run() {
		$this->_hooks->run();
	}

	/**
	 * Loads the plugin meta data.
	 */
	protected function _loadPluginData() {
		if ( ! function_exists( 'get_plugin_data' ) )
			require_once( ABSPATH . '/wp-admin/includes/plugin.php' );
		$this->_data = get_plugin_data( EDDCM_MAIN_FILE );
	}

    /**
     * Deactivates this plugin.
     *
     * @param callbable|string $arg The notice callback function (that will be hooked on `admin_notices` after
     *                              deactivation, or a string specifying the reason for deactivation. Default: null
     */
    public function deactivate($arg = null)
    {
        // load plugins.php file from WordPress if not loaded
        require_once(ABSPATH . 'wp-admin/includes/plugin.php');
        // Deactivate
        \deactivate_plugins(EDDCM_PLUGIN_BASENAME);

        // Arg is null, we are done
        if (is_null($arg)) {
            return;
        }
        // Get the message from the arg
        $message = is_callable($arg)
            ? call_user_func($arg)
            : $arg;
        $title = __('EDD Checkout Message has been deactivated!', 'eddcm');
        $fullMessage = sprintf('<h1>%s</h1><p>%s</p>', $title, $message);
        // Show wp_die screen with back link
        wp_die($fullMessage, $title, array(
            'back_link' => true
        ));
    }

	/**
	 * Alias for self::getInstance()
	 * 
	 * @return self
	 */
	public static function instance() {
		return self::getInstance();
	}

	/**
	 * Gets the singleton instance.
	 * 
	 * @return self
	 */
	public static function getInstance() {
		return ( self::$_instance === null )? new self : self::$_instance;
	}

	/**
	 * Renders a view.
	 * 
	 * @return string the rendered view
	 */
	public static function renderView( $view, $viewbag = array() ) {
		ob_start();
		$viewbag = (object) $viewbag;
		include sprintf('%s/%sView.php', self::VIEWS_PATH, $view );
		return ob_get_clean();
	}

}
